
# q-expansion of newform 768.3.e.i, downloaded from the LMFDB on 02 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 768
weight = 3
poly_data = [9, 0, 8, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0, 0], 1], [[0, -10, 0, -2], 3], [[0, 22, 0, 2], 3], [[12, 5, 3, 1], 3]]

hecke_ring_character_values = [[511, [1, 0, 0, 0]], [517, [1, 0, 0, 0]], [257, [-1, 0, 0, 0]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 1], [0, -2, 0, 0], [-4, 0, 0, 0], [0, 3, 0, 0], [0, -2, 0, -4], [0, 0, 2, 0], [0, -1, 0, -2], [0, 0, 4, 0], [0, -6, 0, 0], [-4, 0, 0, 0], [0, -10, 0, -20], [0, 0, -4, 0], [0, -1, 0, -2], [0, 0, 0, 0], [0, -18, 0, 0], [0, 17, 0, 0], [0, -18, 0, -36], [0, 9, 0, 18], [0, 0, -12, 0], [6, 0, 0, 0], [124, 0, 0, 0], [0, 1, 0, 0], [0, 0, 14, 0], [118, 0, 0, 0], [0, 22, 0, 0], [108, 0, 0, 0], [0, -51, 0, 0], [0, -10, 0, -20], [0, 0, 12, 0], [76, 0, 0, 0], [0, -5, 0, 0], [0, 0, 0, 0], [0, -23, 0, -46], [0, -66, 0, 0], [60, 0, 0, 0], [0, 22, 0, 44], [0, 55, 0, 110], [0, 0, 44, 0], [0, 66, 0, 0], [0, -113, 0, 0], [0, 22, 0, 44], [0, 0, 8, 0], [-102, 0, 0, 0], [0, 86, 0, 0], [188, 0, 0, 0], [0, -47, 0, -94], [188, 0, 0, 0], [0, 137, 0, 0], [0, 46, 0, 92], [0, 0, 14, 0], [0, 0, -48, 0], [122, 0, 0, 0], [0, -57, 0, 0], [0, 0, 60, 0], [0, 0, -28, 0], [0, 18, 0, 0], [348, 0, 0, 0], [0, 46, 0, 92], [0, 0, 42, 0], [0, 17, 0, 34], [0, 6, 0, 0], [0, -87, 0, -174], [0, 0, -20, 0], [562, 0, 0, 0], [0, 2, 0, 0], [0, -79, 0, -158], [-50, 0, 0, 0], [0, 127, 0, 0], [0, 22, 0, 44], [0, 0, -24, 0], [0, 0, -44, 0], [-228, 0, 0, 0], [0, -26, 0, -52], [0, 31, 0, 62], [0, 0, -24, 0], [0, -34, 0, 0], [0, -10, 0, -20], [0, 0, -18, 0], [158, 0, 0, 0], [0, 117, 0, 0], [0, -98, 0, -196], [0, 0, 48, 0], [86, 0, 0, 0], [156, 0, 0, 0], [0, 75, 0, 0], [0, 0, 66, 0], [-194, 0, 0, 0], [0, -198, 0, 0], [-404, 0, 0, 0], [0, -235, 0, 0], [0, 0, -32, 0], [-500, 0, 0, 0], [0, 41, 0, 0], [0, -63, 0, -126], [0, 0, 60, 0], [0, 114, 0, 0], [0, 0, 24, 0], [0, -105, 0, -210], [0, -146, 0, -292], [0, 95, 0, 190], [0, 186, 0, 0], [0, -3, 0, 0], [0, 0, -48, 0], [0, 1, 0, 2], [-18, 0, 0, 0], [0, -43, 0, 0], [0, 0, 96, 0], [0, 0, -92, 0], [358, 0, 0, 0], [-884, 0, 0, 0], [0, -74, 0, -148], [0, 0, -14, 0], [0, 129, 0, 258], [428, 0, 0, 0], [0, 0, -106, 0], [0, -161, 0, -322], [0, 0, 60, 0], [0, -390, 0, 0], [0, 327, 0, 0], [0, -82, 0, -164], [566, 0, 0, 0], [0, -282, 0, 0], [0, 143, 0, 0], [0, 23, 0, 46], [0, -118, 0, 0], [0, 70, 0, 140], [0, 0, -136, 0], [-1140, 0, 0, 0], [0, 70, 0, 140], [0, 1, 0, 2], [0, 0, -100, 0], [-1108, 0, 0, 0], [0, 198, 0, 396], [0, 0, 172, 0], [538, 0, 0, 0], [0, 22, 0, 0], [0, 279, 0, 558], [0, 442, 0, 0], [0, 0, -60, 0], [0, 263, 0, 526], [0, 166, 0, 0], [-1268, 0, 0, 0], [0, 405, 0, 0], [0, 230, 0, 460], [0, 0, 164, 0], [0, 198, 0, 396], [0, 0, -48, 0], [0, -97, 0, -194], [0, 0, -72, 0], [0, 254, 0, 508], [0, 0, 100, 0], [0, 79, 0, 158], [0, 0, 156, 0], [0, -113, 0, -226], [0, 0, -48, 0], [780, 0, 0, 0], [0, 0, 182, 0], [-566, 0, 0, 0], [0, -150, 0, 0], [0, -165, 0, 0], [0, 0, -120, 0], [-692, 0, 0, 0], [0, 159, 0, 0], [0, 0, -118, 0], [0, 0, 44, 0], [-340, 0, 0, 0], [0, 294, 0, 588]]
